# check docs for search terms

# "Precision Medicine" AND 
# "Digital Health" AND 
# "explainable artificial intelligence" OR 
# "interpretable machine learning"

# load libraries

library(topicmodels)
library(pdftools)
library(tm)
library(tidytext)
library(tidyverse)
library(tidylda)

# if (!require("pacman")) install.packages("pacman") 
# pacman::p_load_gh("trinker/textreadr")

# library(textreadr) # no longer active, but can use scripts as needed
library(readtext)
library(scales)
library(udpipe)

# create list of all articles and convert them to text

data_files = list.files(path = "/Users/benallen/Downloads/pdf_text/", full.names = T, recursive = T, pattern = "*docx")

extracted_texts = readtext(data_files, docvarsfrom = "filepaths", dvsep = "/")

num_files = nrow(extracted_texts)

#start preprocessing

#create corpus from vector
# changed from Corpus to VCorpus to allow for a custom tokenizer

docs = VCorpus(DataframeSource(extracted_texts)) 

# remove newlines that follow a hyphen 
remove_newline_hypens = function(x) gsub("-\n", "", x)

# remove newlines that dont follow a period 
remove_newline_wo_periods = function(x)  gsub("(?<!\\.)\n", " ", x, perl = TRUE)

removeDashs = function(x) gsub("[\u2012\u2015\u2013\u2014\u002D\u2212]", " ", x)

remove_special_char = function(x) gsub("[\u00A1\u00A2\u00A3\u00A4\u00A5\u00A6\u00A7\u00A8\u00A9\u00AA\u00AB\u00AC\u00AE\u00AF\u00B0\u00B1\u00B2\u00B3\u00B4\u00B5\u00B6\u00B7\u00B8\u00B9\u00BA\u00BB\u00BC\u00BD\u00BE]", " ", x)

remove_parentheses = function(x) gsub("[()]", "", x)
remove_brackets = function(x) gsub("\\[|\\]", "", x)
remove_vert_bar = function(x) gsub("\\|", "", x)

remove_spec_chars_2 = function(x) gsub('[\\]', '',
     gsub('[/]', '',
          gsub('[-]', '', 
               gsub('[:]', '',  
                    gsub('[*]', '',  
                         gsub('[#]', '',  
                              gsub('[%]', '',  
                                   gsub('[>]', '',  
                                        gsub('[&]', '',
                                             gsub('[{]', '',
                                                  gsub('[}]', '', x)))))))))))


docs = tm_map(docs, content_transformer(tolower))
docs = tm_map(docs, content_transformer(remove_newline_hypens))
docs = tm_map(docs, content_transformer(remove_newline_wo_periods))
docs = tm_map(docs, removeWords, stopwords("english"))
docs = tm_map(docs, content_transformer(removePunctuation))
docs = tm_map(docs, content_transformer(removeNumbers))
docs = tm_map(docs, content_transformer(removeDashs))
docs = tm_map(docs, content_transformer(remove_special_char))
docs = tm_map(docs, content_transformer(remove_spec_chars_2))
docs = tm_map(docs, content_transformer(remove_parentheses))
docs = tm_map(docs, content_transformer(remove_brackets))
docs = tm_map(docs, content_transformer(remove_vert_bar))
docs = tm_map(docs, content_transformer(stripWhitespace))

# check for keywords

# check docs for search terms

# "precision medicine" AND 
# "digital health" AND 
# "explainable artificial intelligence" OR 
# "interpretable machine learning"

## uni-gram and bi-gram and tri-gram keywords

# search for unigrams

NLP_unigram_tokenizer = function(x) {
  unlist(lapply(ngrams(words(x), 1), paste, collapse = "_"), use.names = FALSE)
}

control_list_ngram = list(tokenize = NLP_unigram_tokenizer,
                          weighting = function(x)
                            weightTf(x)
)

# create document term matrix
dtm_unigram = DocumentTermMatrix(docs, control_list_ngram)
dim(dtm_unigram)

# check dtm for blank rows

raw.sum=apply(dtm_unigram,1,FUN=sum) #sum by raw each raw of the table
length(raw.sum[raw.sum == 0])

rownames(as.data.frame(raw.sum[raw.sum == 0]))

dtm_unigram.df = as.data.frame(as.matrix(dtm_unigram))

# remove papers without search terms in the body

# check for different versions of keywords after all docs are in

# precis_medicin 

# digit_health 

# explain_artifici_intellig 

table(dtm_unigram.df$explainable > 0)


# interpret_machin_learn

table(dtm_unigram.df$interpretable > 0)

search_terms = (
  (dtm_unigram.df$precis_medic + 
     dtm.bigrams.df$precis_medicin + 
     dtm.bigrams.df$precis_medicinebas + 
     dtm.bigrams.df$precis_medicinehealth + 
     dtm.bigrams.df$precis_medicineori + 
     dtm.bigrams.df$precis_medicinerel) * 
    (dtm.bigrams.df$digit_health + 
       dtm.bigrams.df$digit_healthb + 
       dtm.bigrams.df$digit_healthcar + 
       dtm.bigrams.df$digit_healthen + 
       dtm.bigrams.df$digit_healthi + 
       dtm.bigrams.df$digit_healthinto + 
       dtm.bigrams.df$digit_healthrel + 
       dtm.bigrams.df$digit_healthsolut + 
       dtm.bigrams.df$digit_healthtitleabstract) * 
    (dtm_unigram.df$explain_artifici + 
       dtm.bigrams.df$interpret_machin + 
       dtm.bigrams.df$interpret_machinelearn + 
       dtm.bigrams.df$interpret_machinelearningderiv) > 0 )


## bi-gram and tri-gram keywords

# search for bigrams

NLP_bigram_tokenizer = function(x) {
  unlist(lapply(ngrams(words(x), 2), paste, collapse = "_"), use.names = FALSE)
}

control_list_ngram = list(tokenize = NLP_bigram_tokenizer,
                          weighting = function(x)
                            weightTf(x)
)

# create document term matrix
dtm_bigram = DocumentTermMatrix(docs, control_list_ngram)
dim(dtm_bigram)

# check dtm for blank rows

raw.sum=apply(dtm_bigram,1,FUN=sum) #sum by raw each raw of the table
 length(raw.sum[raw.sum == 0])

rownames(as.data.frame(raw.sum[raw.sum == 0]))

dtm_bigram.df = as.data.frame(as.matrix(dtm_bigram))

# remove papers without search terms in the body

# check for different versions of keywords after all docs are in

# precis_medicin 

table(dtm_bigram.df$precision_medicine)
table(dtm_bigram.df$precision_medicinebased)

# digit_health

table(dtm_bigram.df$digital_health)
table(dtm_bigram.df$digital_healthcare)

# explain_artifici_intellig 

table(dtm_bigram.df$explainable_artificial)
table(dtm_bigram.df$explainable_ai)

# interpret_machin_learn

table(dtm_bigram.df$interpretable_machine)
table(dtm_bigram.df$interpretable_machinelearning)

#####

xai_iml = (dtm_bigram.df$interpretable_machine + 
             dtm_bigram.df$interpretable_machinelearning + 
             dtm_bigram.df$interpretable_artificial + 
             dtm_bigram.df$interpretable_ai + 
             dtm_bigram.df$explainable_artificial + 
             dtm_bigram.df$explainable_ai + 
             dtm_bigram.df$explainable_aicdss + 
             dtm_bigram.df$explainable_aiomt + 
             dtm_bigram.df$explainable_aiot +  
             dtm_bigram.df$explainable_atrtificial) > 0

precision_medicine = (dtm_bigram.df$precision_medicine + 
                        dtm_bigram.df$precision_medicinebased + 
                        dtm_bigram.df$precision_medicinehealth + 
                        dtm_bigram.df$precision_medicineoriented + 
                        dtm_bigram.df$precision_medicinerelated + 
                        dtm_bigram.df$precisionmedicine_contact + 
                        dtm_bigram.df$precisionmedicine_treatments) > 0

digital_health = (dtm_bigram.df$digital_health + 
                    dtm_bigram.df$digital_healthcare + 
                    dtm_bigram.df$digital_healthbe + 
                    dtm_bigram.df$digital_healthenabled +  
                    dtm_bigram.df$digital_healthinto + 
                    dtm_bigram.df$digital_healthis + 
                    dtm_bigram.df$digital_healthsolutions + 
                    dtm_bigram.df$digital_healthtitleabstract + 
                    dtm_bigram.df$digitalhealth_needstobe + 
                    dtm_bigram.df$digitalhealth_workforce + 
                    dtm_bigram.df$digitalhealth_isincreasingly + 
                    dtm_bigram.df$digitalhealthtu_dresdendeinfrastructureregulatoryaffairs +  
                    dtm_bigram.df$digitalhealthcanbedescribedasastrategytotransformthequality_health) > 0

search_terms = xai_iml == TRUE & digital_health == TRUE & precision_medicine == TRUE

revised_data_files = data_files[search_terms]


# create document term matrix
dtm_bigram = DocumentTermMatrix(docs, control_list_ngram)
dim(dtm_bigram)

dtm_bigram_revised = DocumentTermMatrix(docs[search_terms], control_list_ngram)

dim(dtm_bigram_revised)

# copy files with matching terms to new folder

file.copy(from=revised_data_files, to="/Users/benallen/Documents/ai_digital_health/pdf_text_matching_terms2/", 
          overwrite = FALSE, 
          recursive = FALSE, 
          copy.mode = TRUE)


revised_data_files = list.files(path = "/Users/benallen/Documents/ai_digital_health/pdf_text_matching_terms2/", full.names = T, recursive = T, pattern = "*docx")

extracted_texts = readtext(revised_data_files, docvarsfrom = "filepaths", dvsep = "/")

num_files = nrow(extracted_texts)

#start preprocessing

#create corpus from vector
# changed from Corpus to VCorpus to allow for a custom tokenizer

docs = VCorpus(DataframeSource(extracted_texts)) 

# remove newlines that follow a hyphen 
remove_newline_hypens = function(x) gsub("-\n", "", x)

# remove newlines that dont follow a period 
remove_newline_wo_periods = function(x)  gsub("(?<!\\.)\n", " ", x, perl = TRUE)

removeDashs = function(x) gsub("[\u2012\u2015\u2013\u2014\u002D\u2212]", " ", x)

remove_special_char = function(x) gsub("[\u00A1\u00A2\u00A3\u00A4\u00A5\u00A6\u00A7\u00A8\u00A9\u00AA\u00AB\u00AC\u00AE\u00AF\u00B0\u00B1\u00B2\u00B3\u00B4\u00B5\u00B6\u00B7\u00B8\u00B9\u00BA\u00BB\u00BC\u00BD\u00BE\u005B\u005D\u0028\u0029\u007C\u2768\u2769\u276A\u276B]", " ", x)

remove_parentheses = function(x) gsub("[()]", "", x)
remove_brackets = function(x) gsub("\\[|\\]", "", x)
remove_vert_bar = function(x) gsub("\\|", "", x)

remove_spec_chars_2 = function(x) gsub('[\\]', '',
                                       gsub('[/]', '',
                                            gsub('[-]', '', 
                                                 gsub('[:]', '',  
                                                      gsub('[*]', '',  
                                                           gsub('[#]', '',  
                                                                gsub('[%]', '',  
                                                                     gsub('[>]', '',  
                                                                          gsub('[&]', '',
                                                                               gsub('[{]', '',
                                                                                    gsub('[}]', '', x)))))))))))

docs = tm_map(docs, content_transformer(remove_newline_hypens))
docs = tm_map(docs, content_transformer(remove_newline_wo_periods))
docs = tm_map(docs, content_transformer(removeNumbers))
docs = tm_map(docs, content_transformer(removeDashs))
docs = tm_map(docs, content_transformer(remove_special_char))
docs = tm_map(docs, content_transformer(remove_spec_chars_2))
docs = tm_map(docs, content_transformer(remove_parentheses))
docs = tm_map(docs, content_transformer(remove_brackets))
docs = tm_map(docs, content_transformer(remove_vert_bar))

for (i in 1:length(docs)){
  
  writeLines(
    docs[[i]][["content"]],
    paste("/Users/benallen/Documents/ai_digital_health/txt_files_matching_terms/", docs[[i]][["meta"]][["id"]], "_clean.txt", sep = ""))
  
}


# write each paragraph of each paper as separate txt file


for (i in 1:length(revised_data_files)){
  paper = read_document(file = revised_data_files[i])
  for (j in 1:length(paper)){
    write.table(
      paper[j], 
      file = paste("paper_", i, "_", j, ".txt", sep=""), 
      row.names = FALSE, col.names = FALSE
    )
  }
}

library(officer)
library(tidyverse)
library(readtext)
library(tm)
library(tidytext)
library(scales)
library(tidylda)
library(topicmodels)
library(textmineR)
library(LDAvis)
library(SnowballC)

# write each paragraph of each paper as separate txt file

setwd('/Users/benallen/Documents/ai_digital_health/cleaned_final_27/')

full_paper_docx_files = list.files(path = "/Users/benallen/Documents/ai_digital_health/cleaned_final_27/", full.names = T, recursive = T, pattern = "*docx")

# Create a new directory for paragraph files
dir.create("/Users/benallen/Documents/ai_digital_health/clean_27_txt_paragraph_files")

## read in each paper and save each paragraph as a separate file

setwd('/Users/benallen/Documents/ai_digital_health/clean_27_txt_paragraph_files/')

for (i in 1:length(full_paper_docx_files)){
  
  paper = read_docx(full_paper_docx_files[i])
  paper = docx_summary(paper)
  
  for (j in 1:nrow(paper)){
    write.table(
      paper$text[j], 
      file = paste("paper_", i, "_", j, ".txt", sep=""),
      row.names = FALSE, col.names = FALSE, quote = FALSE
    )
  }
}


cleaned_txt_paragraph_files = list.files(path = "/Users/benallen/Documents/ai_digital_health/clean_27_txt_paragraph_files/", full.names = T, recursive = T, pattern = "*txt")

extracted_texts = readtext(cleaned_txt_paragraph_files, docvarsfrom = "filepaths", dvsep = "/")

num_files = nrow(extracted_texts)

## start pre processing

# clean text a bit

#create corpus from vector
# changed from Corpus to VCorpus to allow for a custom tokenizer

docs = VCorpus(DataframeSource(extracted_texts)) 

docs <- tm_map(docs, removeWords, stopwords("english"))
docs <- tm_map(docs, stemDocument, language = "english")  
docs <- tm_map(docs, removePunctuation)

# save cleaned text as new paragraphs

for (i in 1:length(docs)){
  
  writeLines(
    docs[[i]][["content"]],
    paste("/Users/benallen/Documents/ai_digital_health/clean_stemmed_27_txt_paragraph_files/", docs[[i]][["meta"]][["id"]], "_stemmed.txt", sep = ""))
  
}

# read in stemmed paragraphs

stemmed_txt_paragraph_files = list.files(path = "/Users/benallen/Documents/ai_digital_health/clean_stemmed_27_txt_paragraph_files/", full.names = T, recursive = T, pattern = "*txt")

extracted_texts = readtext(stemmed_txt_paragraph_files, docvarsfrom = "filepaths", dvsep = "/")

num_files = nrow(extracted_texts)


pm_dm_xai_ngrams <- extracted_texts %>% 
  unnest_tokens(ngram, text, token = "ngrams", n = 3, n_min = 1)

ngram_counts <- pm_dm_xai_ngrams |> 
  count(doc_id, ngram, sort = TRUE) |> 
  filter(!is.na(ngram))

tidy_ngrams = ngram_counts |> 
  bind_tf_idf(ngram, doc_id, n) |> 
  arrange(desc(tf_idf))

ngram_dtm = tidy_ngrams %>% 
  cast_dtm(doc_id, ngram, n)

dim(ngram_dtm)

# remove paragraphs from dtm with less than 5 words

rowTotals <- apply(ngram_dtm , 1, sum) #Find the sum of words in each Document
dtm.new   <- ngram_dtm[rowTotals > 5, ]           #remove all docs with less than 5 words
dim(dtm.new)


# Remove terms with less than 2 occurrences
# 0.95 means terms with less than 2 occurrences will be removed
dtm_filtered <- removeSparseTerms(dtm.new, sparse = 0.95)  

dim(dtm_filtered)

# find top terms
popular_terms = colnames(dtm_filtered)
write.csv(x = popular_terms, file = "/Users/benallen/Downloads/popular_terms4.csv")


# Display the filtered document term matrix
print("Filtered Document Term Matrix:")
inspect(dtm_filtered)

dim(dtm.new)
dim(dtm_filtered)



# dtm went from 241027 ngrams to 273

# ------

## now that the dtm is finished and cleaned
## split it up and do coherence on data subsets

# break into 10 folds

folds = 10
num_files = nrow( dtm_filtered )
size=round( nrow(dtm_filtered )/ folds )
starts = seq( 1, num_files, by=size )

i = starts[1]

size=round( nrow(dtm_filtered )/ folds ) + 1
set.seed(23322)
rand_rows = data.frame(
  row = sample( x = dtm_filtered$nrow, size = dtm_filtered$nrow, replace = FALSE),
  fold = rep( 1:folds, ceiling(num_files / folds) )[1:num_files])

# Look at coherence averaged across all folds

coherence_results = data.frame(matrix(nrow = 0, ncol = 0))

folds = 1:10

for (i in 1:length(folds)){
  
  fold = rand_rows$row[rand_rows$fold == i]
  dtm.train = dtm_filtered[-fold,]
  
  temp_coherence = ldatuning::FindTopicsNumber(
    dtm = dtm.train, topics = 2:20, 
    metrics = c("Griffiths2004", "CaoJuan2009", "Arun2010", "Deveaud2014"),
    method = "Gibbs",
    control = list(seed = 23322))
  
  coherence_results = bind_rows(coherence_results, temp_coherence)
  
}

# rescale metrics to same scale and direction

coherence_results_z = coherence_results

# coherence_results_z$Griffiths2004 = 1 + ((rescale(coherence_results_z$Griffiths2004)) * - 1)
coherence_results_z$Griffiths2004 = rescale(coherence_results_z$Griffiths2004)

coherence_results_z$CaoJuan2009 = rescale(coherence_results_z$CaoJuan2009)

coherence_results_z$Arun2010 = rescale(coherence_results_z$Arun2010)

coherence_results_z$Deveaud2014 = rescale(coherence_results_z$Deveaud2014)


# coherence_results_z$united = (coherence_results_z[,2] + coherence_results_z[,3] + coherence_results_z[,4]) / 3

avg_coherence_measures = data.frame(
  Griffiths2004 = aggregate(x = coherence_results_z$Griffiths2004, list(coherence_results_z$topics), FUN=median),
  CaoJuan2009 = aggregate(x = coherence_results_z$CaoJuan2009, list(coherence_results_z$topics), FUN=median),
  Arun2010 = aggregate(x = coherence_results_z$Arun2010, list(coherence_results_z$topics), FUN=median),
  Deveaud2014 = aggregate(x = coherence_results_z$Deveaud2014, list(coherence_results_z$topics), FUN=median)
)

avg_coherence_measures = select(avg_coherence_measures, -c(Arun2010.Group.1, CaoJuan2009.Group.1, Deveaud2014.Group.1))

colnames(avg_coherence_measures)[1] = 'topics'

ldatuning::FindTopicsNumber_plot(avg_coherence_measures)

## need to find intersection of line pairs

avg_coherence_measures_long = avg_coherence_measures %>% pivot_longer(
  cols=c( 'Griffiths2004.x', 'CaoJuan2009.x', 'Arun2010.x', 'Deveaud2014.x'), 
  names_to='metric',
  values_to='coherence')

ggplot(data = avg_coherence_measures_long, aes(x = topics, y = coherence, fill = metric) ) + geom_line(aes(color = metric)) + scale_x_continuous(n.breaks=19)

# find intersection of all lines
# add 1 to each to account for topics starting at 2


# minimize Arun2010 and maximize Deveaud2014
ad = min(which(avg_coherence_measures$Arun2010.x < avg_coherence_measures$Deveaud2014.x)) + 1

# minimize Arun2010 and maximize Griffiths2004
ag = min(which(avg_coherence_measures$Arun2010.x < avg_coherence_measures$Griffiths2004.x)) + 1

#  minimize CaoJuan2009 and maximize Deveaud2014
# cd = min(which(avg_coherence_measures$Deveaud2014.x < avg_coherence_measures$CaoJuan2009.x)) + 1

#  minimize CaoJuan2009 and maximize Griffiths2004
cg = min(which(avg_coherence_measures$Griffiths2004.x < avg_coherence_measures$CaoJuan2009.x)) + 1


optimal_coherence = median(ad, ag, cg)

optimal_coherence

## run topic model

# number of topics

K <- optimal_coherence

burnin = 100
iter = 1000
keep = 50

# Fit a Latent Dirichlet Allocation model
# note the number of topics is arbitrary here
# see extensions for more info

dtm.dgCMatrix = Matrix::sparseMatrix(
  i=dtm_filtered$i, 
  j=dtm_filtered$j, 
  x=dtm_filtered$v,
  dims=c(dtm_filtered$nrow, dtm_filtered$ncol),
  dimnames = dtm_filtered$dimnames)

set.seed(23322)
model <- FitLdaModel(dtm = dtm.dgCMatrix, 
                     k = K,
                     iterations = 1000, 
                     burnin = 100,
                     alpha = 0.1,
                     beta = 0.05,
                     optimize_alpha = TRUE,
                     calc_likelihood = TRUE,
                     calc_coherence = TRUE,
                     calc_r2 = TRUE,
                     cpus = 7) 

# log Likelihood (does not consider the prior) 
plot(model$log_likelihood, type = "l")

hist(model$coherence, col="blue", main = "Histogram of coherence")

# get to ten terms per topic

model_top_terms = GetTopTerms(model$phi, 10, return_matrix = TRUE)

write.csv(
  t(as.data.frame(model_top_terms)), 
  file = "/Users/benallen/Documents/ai_digital_health/model_top_terms_11292023.csv",
  quote = FALSE, 
  row.names = TRUE)

t(as.data.frame(model_top_terms))

## redo coherene test after removing non-meaningful terms

# Remove the specified terms from the Document-Term Matrix

dtm_filtered_2 <- dtm_filtered[, !(colnames(dtm_filtered) %in% 'al')]
dtm_filtered_2 <- dtm_filtered_2[, !(colnames(dtm_filtered_2) %in% 'can')]
dtm_filtered_2 <- dtm_filtered_2[, !(colnames(dtm_filtered_2) %in% 'e')]
dtm_filtered_2 <- dtm_filtered_2[, !(colnames(dtm_filtered_2) %in% 'et')]
dtm_filtered_2 <- dtm_filtered_2[, !(colnames(dtm_filtered_2) %in% 'et al')]
dtm_filtered_2 <- dtm_filtered_2[, !(colnames(dtm_filtered_2) %in% 'the')]
dtm_filtered_2 <- dtm_filtered_2[, !(colnames(dtm_filtered_2) %in% 'use')]
dtm_filtered_2 <- dtm_filtered_2[, !(colnames(dtm_filtered_2) %in% 'includ')]
dtm_filtered_2 <- dtm_filtered_2[, !(colnames(dtm_filtered_2) %in% 'in')]

# check dtm for blank rows

raw.sum=apply(dtm_filtered_2,1,FUN=sum) #sum by raw each raw of the table
length(raw.sum[raw.sum == 0])

rownames(as.data.frame(raw.sum[raw.sum == 0]))

doc_to_remove = "paper_7_20.txt_stemmed.txt"

# remove blank row

dtm_filtered_2 = dtm_filtered_2[!(rownames(dtm_filtered_2) %in% doc_to_remove), , drop = FALSE]

dim(dtm_filtered)
# 1733  273

dim(dtm_filtered_2)
# 1732  264

# rerun coherence test

## now that the dtm is finished and cleaned
## split it up and do coherence on data subsets

# break into 10 folds

folds = 10
num_files = nrow( dtm_filtered_2 )
size=round( nrow(dtm_filtered_2 )/ folds )
starts = seq( 1, num_files, by=size )

i = starts[1]

size=round( nrow(dtm_filtered_2 )/ folds ) + 1
set.seed(23322)
rand_rows = data.frame(
  row = sample( x = dtm_filtered_2$nrow, size = dtm_filtered_2$nrow, replace = FALSE),
  fold = rep( 1:folds, ceiling(num_files / folds) )[1:num_files])

# Look at coherence averaged across all folds

coherence_results = data.frame(matrix(nrow = 0, ncol = 0))

folds = 1:10

for (i in 1:length(folds)){
  
  fold = rand_rows$row[rand_rows$fold == i]
  dtm.train = dtm_filtered_2[-fold,]
  
  temp_coherence = ldatuning::FindTopicsNumber(
    dtm = dtm.train, topics = 2:20, 
    metrics = c("Griffiths2004", "CaoJuan2009", "Arun2010", "Deveaud2014"),
    method = "Gibbs",
    control = list(seed = 23322))
  
  coherence_results = bind_rows(coherence_results, temp_coherence)
  
}

# rescale metrics to same scale and direction

coherence_results_z = coherence_results

# coherence_results_z$Griffiths2004 = 1 + ((rescale(coherence_results_z$Griffiths2004)) * - 1)
coherence_results_z$Griffiths2004 = rescale(coherence_results_z$Griffiths2004)

coherence_results_z$CaoJuan2009 = rescale(coherence_results_z$CaoJuan2009)

coherence_results_z$Arun2010 = rescale(coherence_results_z$Arun2010)

coherence_results_z$Deveaud2014 = rescale(coherence_results_z$Deveaud2014)


# coherence_results_z$united = (coherence_results_z[,2] + coherence_results_z[,3] + coherence_results_z[,4]) / 3

avg_coherence_measures = data.frame(
  Griffiths2004 = aggregate(x = coherence_results_z$Griffiths2004, list(coherence_results_z$topics), FUN=median),
  CaoJuan2009 = aggregate(x = coherence_results_z$CaoJuan2009, list(coherence_results_z$topics), FUN=median),
  Arun2010 = aggregate(x = coherence_results_z$Arun2010, list(coherence_results_z$topics), FUN=median),
  Deveaud2014 = aggregate(x = coherence_results_z$Deveaud2014, list(coherence_results_z$topics), FUN=median)
)

avg_coherence_measures = select(avg_coherence_measures, -c(Arun2010.Group.1, CaoJuan2009.Group.1, Deveaud2014.Group.1))

colnames(avg_coherence_measures)[1] = 'topics'

ldatuning::FindTopicsNumber_plot(avg_coherence_measures)

## need to find intersection of line pairs

avg_coherence_measures_long = avg_coherence_measures %>% pivot_longer(
  cols=c( 'Griffiths2004.x', 'CaoJuan2009.x', 'Arun2010.x', 'Deveaud2014.x'), 
  names_to='metric',
  values_to='coherence')

ggplot(data = avg_coherence_measures_long, aes(x = topics, y = coherence, fill = metric) ) + geom_line(aes(color = metric)) + scale_x_continuous(n.breaks=19)

# find interseciton of all lines
# add 1 to each to account for topics starting at 2


# minimize Arun2010 and maximize Deveaud2014
ad = min(which(avg_coherence_measures$Arun2010.x < avg_coherence_measures$Deveaud2014.x)) + 1

# minimize Arun2010 and maximize Griffiths2004
ag = min(which(avg_coherence_measures$Arun2010.x < avg_coherence_measures$Griffiths2004.x)) + 1

#  minimize CaoJuan2009 and maximize Deveaud2014
# cd = min(which(avg_coherence_measures$Deveaud2014.x < avg_coherence_measures$CaoJuan2009.x)) + 1

#  minimize CaoJuan2009 and maximize Griffiths2004
cg = min(which(avg_coherence_measures$Griffiths2004.x < avg_coherence_measures$CaoJuan2009.x)) + 1


optimal_coherence = median(ad, ag, cg)

optimal_coherence

###

# Finally the model looks good!

# now for interpretaiton

burnin = 100
iter = 1000
keep = 50

# Fit a Latent Dirichlet Allocation model
# note the number of topics is arbitrary here
# see extensions for more info

dtm.dgCMatrix = Matrix::sparseMatrix(
  i=dtm_filtered_2$i, 
  j=dtm_filtered_2$j, 
  x=dtm_filtered_2$v,
  dims=c(dtm_filtered_2$nrow, dtm_filtered_2$ncol),
  dimnames = dtm_filtered_2$dimnames)

set.seed(23322)
model <- FitLdaModel(dtm = dtm.dgCMatrix, 
                     k = optimal_coherence,
                     iterations = 1000, 
                     burnin = 100,
                     alpha = 0.1,
                     beta = 0.05,
                     optimize_alpha = TRUE,
                     calc_likelihood = TRUE,
                     calc_coherence = TRUE,
                     calc_r2 = TRUE,
                     cpus = 7) 

# log Likelihood (does not consider the prior) 
plot(model$log_likelihood, type = "l")

hist(model$coherence, col="blue", main = "Histogram of coherence")

# get to ten terms per topic

model_top_terms = GetTopTerms(model$phi, 10, return_matrix = TRUE)

write.csv(
  t(as.data.frame(model_top_terms)), 
  file = "/Users/benallen/Documents/ai_digital_health/model_top_terms_011292023.csv",
  quote = FALSE, 
  row.names = TRUE)

# find the top 5 terms for each topic

model_top_5_terms = GetTopTerms(model$phi, 5, return_matrix = TRUE)

write.csv(
  t(as.data.frame(model_top_5_terms)), 
  file = "/Users/benallen/Documents/ai_digital_health/model_top_5_terms_011292023.csv",
  quote = FALSE, 
  row.names = TRUE)


top_5_papers = paste(topic_1, topic_2, topic_3, topic_4, topic_5)
write.csv(top_5_papers, "/Users/benallen/Documents/ai_digital_health/top_5_paper_per.csv")

paper_2_thetas = model$theta[grep("paper_2_", rownames(model$theta)),]
write.csv(paper_2_thetas, "/Users/benallen/Documents/ai_digital_health/paper_2_thetas.csv")
# find the prevalence of each topic across the documents

prevalence = as.data.frame(colSums(model$theta) / sum(model$theta) * 100)
colnames(prevalence)[1] = "prevalence"
prevalence$topic_number = rownames(prevalence)
prevalence

prevalence = prevalence[order(-prevalence$prevalence),]

ggplot(prevalence, aes(x = topic_number[order], y = prevalence)) + geom_boxplot()

#

# find top paragraphs for each topic

topic_1 = rownames(model$theta)[order(-model$theta[,1])][1:10]
topic_2 = rownames(model$theta)[order(-model$theta[,2])][1:10]
topic_3 = rownames(model$theta)[order(-model$theta[,3])][1:10]
topic_4 = rownames(model$theta)[order(-model$theta[,4])][1:10]
topic_5 = rownames(model$theta)[order(-model$theta[,5])][1:10]


topic_1_orig_txt = gsub("_stemmed.txt", "", topic_1)
topic_2_orig_txt = gsub("_stemmed.txt", "", topic_2)
topic_3_orig_txt = gsub("_stemmed.txt", "", topic_3)
topic_4_orig_txt = gsub("_stemmed.txt", "", topic_4)
topic_5_orig_txt = gsub("_stemmed.txt", "", topic_5)

# Initialize an empty character vector to store the combined text
topic_1_combined_text <- character()

# Read each .txt file and append its content to the vector
for (file_path in topic_1_orig_txt) {
  text <- readLines(file_path, warn = FALSE)
  topic_1_combined_text <- c(topic_1_combined_text, text, " ")
}

# Initialize an empty character vector to store the combined text
topic_2_combined_text <- character()

# Read each .txt file and append its content to the vector
for (file_path in topic_2_orig_txt) {
  text <- readLines(file_path, warn = FALSE)
  topic_2_combined_text <- c(topic_2_combined_text, text, " ")
}

# Initialize an empty character vector to store the combined text
topic_3_combined_text <- character()

# Read each .txt file and append its content to the vector
for (file_path in topic_3_orig_txt) {
  text <- readLines(file_path, warn = FALSE)
  topic_3_combined_text <- c(topic_3_combined_text, text, " ")
}

# Initialize an empty character vector to store the combined text
topic_4_combined_text <- character()

# Read each .txt file and append its content to the vector
for (file_path in topic_4_orig_txt) {
  text <- readLines(file_path, warn = FALSE)
  topic_4_combined_text <- c(topic_4_combined_text, text, " ")
}

# Initialize an empty character vector to store the combined text
topic_5_combined_text <- character()

# Read each .txt file and append its content to the vector
for (file_path in topic_5_orig_txt) {
  text <- readLines(file_path, warn = FALSE)
  topic_5_combined_text <- c(topic_5_combined_text, text, " ")
}

# Create a new Word document and add paragraph text
doc <- read_docx()
doc <- doc %>% body_add(topic_1_combined_text)
print(doc, target="/Users/benallen/Documents/ai_digital_health/Topic_1_summary.docx")

# Create a new Word document and add paragraph text
doc <- read_docx()
doc <- doc %>% body_add(topic_2_combined_text)
print(doc, target="/Users/benallen/Documents/ai_digital_health/Topic_2_summary.docx")

# Create a new Word document and add paragraph text
doc <- read_docx()
doc <- doc %>% body_add(topic_3_combined_text)
print(doc, target="/Users/benallen/Documents/ai_digital_health/Topic_3_summary.docx")

# Create a new Word document and add paragraph text
doc <- read_docx()
doc <- doc %>% body_add(topic_4_combined_text)
print(doc, target="/Users/benallen/Documents/ai_digital_health/Topic_4_summary.docx")

# Create a new Word document and add paragraph text
doc <- read_docx()
doc <- doc %>% body_add(topic_5_combined_text)
print(doc, target="/Users/benallen/Documents/ai_digital_health/Topic_5_summary.docx")




model$theta

head(topic_1)
head(topic_2)
head(topic_3)
head(topic_4)
head(topic_5)

length(model$theta)
length(model$beta)
length()

num_files * length(model$beta)



topic_1$doc = row.names(topic_1)
colnames(topic_1)[1] = "theta"


head(topic_1)

topic_1 = topic_1[topic_1$`model$theta[order(model$theta[, 1]), 1]`]


# need to convert final dtm to a tidy format

new_ngrams = tidy(dtm_filtered_3)

# and then create a word cloud of final dtm

wordcloud(
  words = new_ngrams$term, 
  freq = new_ngrams$count, 
  min.freq = 1,
  max.words=200, 
  random.order=FALSE, 
  rot.per=0.35,
  colors=brewer.pal(8, "Dark2"))

# Set up the plotting device (e.g., png)
png("/Users/benallen/Documents/ai_digital_health/precision_medicine_digital_health_xai_word_cloud.png", width = 1100, height = 1100, res = 300)

# Create a word cloud
wordcloud(
  words = new_ngrams$term, 
  freq = new_ngrams$count, 
  min.freq = 1,
  max.words=200, 
  random.order=FALSE, 
  rot.per=0.35,
  colors=brewer.pal(8, "Dark2"))

# Close the plotting device
dev.off()
